
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: coroutine, IsAddOnLoaded, MainMenuBar, next, QuickKeybindFrame

local AnchorUtil = AnchorUtil
local EditModeManagerFrame = EditModeManagerFrame
local GridLayoutUtil = GridLayoutUtil
local InCombatLockdown = InCombatLockdown
local KeybindFrames_InQuickKeybindMode = KeybindFrames_InQuickKeybindMode
local math = math
local RegisterAttributeDriver = RegisterAttributeDriver
local Settings = Settings
local table = table
local type = type
local unpack = unpack

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Blizzard ActionBar")

M.Enum = {}

M.Enum.ActionBar = {
	ActionBar1 = MainMenuBar,
	ActionBar2 = MultiBarBottomLeft,
	ActionBar3 = MultiBarBottomRight,
	ActionBar4 = MultiBarRight,
	ActionBar5 = MultiBarLeft,
	ActionBar6 = MultiBar5,
	ActionBar7 = MultiBar6,
	ActionBar8 = MultiBar7,
	PetBar = PetActionBar,
	PossessBar = PossessActionBar,
	StanceBar = StanceBar,
}

M.Enum.ActionBarCvar = {
	[M.Enum.ActionBar.ActionBar2] = "PROXY_SHOW_ACTIONBAR_2",
	[M.Enum.ActionBar.ActionBar3] = "PROXY_SHOW_ACTIONBAR_3",
	[M.Enum.ActionBar.ActionBar4] = "PROXY_SHOW_ACTIONBAR_4",
	[M.Enum.ActionBar.ActionBar5] = "PROXY_SHOW_ACTIONBAR_5",
	[M.Enum.ActionBar.ActionBar6] = "PROXY_SHOW_ACTIONBAR_6",
	[M.Enum.ActionBar.ActionBar7] = "PROXY_SHOW_ACTIONBAR_7",
	[M.Enum.ActionBar.ActionBar8] = "PROXY_SHOW_ACTIONBAR_8",
}

M.Enum.ActionBarKey = {
	[M.Enum.ActionBar.ActionBar1] = "1",
	[M.Enum.ActionBar.ActionBar2] = "2",
	[M.Enum.ActionBar.ActionBar3] = "3",
	[M.Enum.ActionBar.ActionBar4] = "4",
	[M.Enum.ActionBar.ActionBar5] = "5",
	[M.Enum.ActionBar.ActionBar6] = "6",
	[M.Enum.ActionBar.ActionBar7] = "7",
	[M.Enum.ActionBar.ActionBar8] = "8",
}

M.Enum.ActionBarLabel = {
	[M.Enum.ActionBar.ActionBar1] = "Action Bar 1",
	[M.Enum.ActionBar.ActionBar2] = "Action Bar 2",
	[M.Enum.ActionBar.ActionBar3] = "Action Bar 3",
	[M.Enum.ActionBar.ActionBar4] = "Action Bar 4",
	[M.Enum.ActionBar.ActionBar5] = "Action Bar 5",
	[M.Enum.ActionBar.ActionBar6] = "Action Bar 6",
	[M.Enum.ActionBar.ActionBar7] = "Action Bar 7",
	[M.Enum.ActionBar.ActionBar8] = "Action Bar 8",
	[M.Enum.ActionBar.PetBar] = "Pet Bar",
	[M.Enum.ActionBar.PossessBar] = "Possess Bar",
	[M.Enum.ActionBar.StanceBar] = "Stance Bar",
}

local HIDE_STATE_TEXT = "hide"
local SHOW_STATE_TEXT = "show"

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	if IsAddOnLoaded("Bartender4") or IsAddOnLoaded("Dominos") then
		return
	end

	if not self:IsHooked(EditModeManagerFrame, "EnterEditMode") then
		self:SecureHook(EditModeManagerFrame, "EnterEditMode", "ConfigureActionBars")
	end

	if not self:IsHooked(EditModeManagerFrame, "ExitEditMode") then
		self:SecureHook(EditModeManagerFrame, "ExitEditMode", "ConfigureActionBars")
	end

	if not self:IsHooked("MultiActionBar_Update") then
		self:SecureHook("MultiActionBar_Update", "ConfigureActionBars")
	end

	if not self:IsHooked(QuickKeybindFrame, "Show") then
		self:SecureHook(QuickKeybindFrame, "Show", "ConfigureActionBars")
	end

	if not self:IsHooked(QuickKeybindFrame, "Hide") then
		self:SecureHook(QuickKeybindFrame, "Hide", "ConfigureActionBars")
	end

	self:RegisterEvent("SETTINGS_LOADED", "ConfigureActionBars")
	self:RegisterMessage("SIXUI_CLUTTER_KEY_CHANGED", "ConfigureAll")
	self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED")
	self:RegisterMessage("SIXUI_CONFIG_UPDATE")
	self:RegisterMessage("SIXUI_LAYOUT_CHANGED", "ConfigureAll")
	self:RegisterMessage("SIXUI_PROFILE_UPDATE", "ConfigureAll")
	self:ConfigureAll()
end

function M:ConfigureAll()
	self:ConfigureActionBars()
	self:ConfigureActionButtons()
	self:ConfigureMainMenuBar()
end

function M:GetBarConfig(bar)
	local layout = A:GetLayout()
	local barKey = self.Enum.ActionBarKey[bar]

	return self.config.barConfigs[layout][barKey]
end

function M:IterateBarConfigs()
	return coroutine.wrap(function ()
		for _, bar in next, {
			M.Enum.ActionBar.ActionBar1,
			M.Enum.ActionBar.ActionBar2,
			M.Enum.ActionBar.ActionBar3,
			M.Enum.ActionBar.ActionBar4,
			M.Enum.ActionBar.ActionBar5,
			M.Enum.ActionBar.ActionBar6,
			M.Enum.ActionBar.ActionBar7,
			M.Enum.ActionBar.ActionBar8,
		} do
			coroutine.yield(bar, self:GetBarConfig(bar), self.Enum.ActionBarLabel[bar])
		end
	end)
end

-------------------------------------------------------------------------------
-- Action Bar
-------------------------------------------------------------------------------

M.text = {}

function M:ConfigureActionBars()
	if InCombatLockdown() then
		A.CombatQueue:Add(self.ConfigureActionBars, self.ConfigureActionBars, self)
	else
		for _, bar in next, self.Enum.ActionBar do
			self:ConfigureActionBar(bar)
		end
	end
end

function M:ConfigureActionBar(bar)
	local barConfig = self:GetBarConfig(bar)

	self:UpdateGridLayout(bar, barConfig)
	self:UpdateVisibility(bar, barConfig)
end

function M:ConfigureMainMenuBar()
	MainMenuBar.BorderArt:SetTexture(nil)
	MainMenuBar.EndCaps:DisableDrawLayer("OVERLAY")
end

function M:ShouldShowActionBar(bar)
	return not M.Enum.ActionBarCvar[bar] or Settings.GetValue(M.Enum.ActionBarCvar[bar])
end

function M:UpdateGridLayout(bar, barConfig)
	if barConfig and bar.isHorizontal then
		if barConfig.reverseRowsOnHorizontalBars then
			local stride = math.ceil(#bar.shownButtonContainers / bar.numRows)
			local buttonPadding = math.max(bar.buttonPadding, 4)
			local layout = GridLayoutUtil.CreateStandardGridLayout(stride, buttonPadding, buttonPadding, 1, -1)

			GridLayoutUtil.ApplyGridLayout(bar.shownButtonContainers, AnchorUtil.CreateAnchor("TOPLEFT", bar, "TOPLEFT"), layout)
		else
			bar.oldGridSettings = nil
			bar:UpdateGridLayout()
		end
	end
end

function M:UpdateVisibility(bar, barConfig)
	if (not barConfig) then return end
	local shouldShow = self:ShouldShowActionBar(bar)
	local stateText = shouldShow and SHOW_STATE_TEXT or HIDE_STATE_TEXT

	if barConfig.stateText ~= "" and not EditModeManagerFrame:IsEditModeActive() and not KeybindFrames_InQuickKeybindMode() and shouldShow then
		stateText = A.FlavourStateText(barConfig.stateText)
	end

	RegisterAttributeDriver(bar, "state-visibility", stateText)
end

-------------------------------------------------------------------------------
-- Action Button
-------------------------------------------------------------------------------

function M:ConfigureActionButtons()
	table.wipe(self.text)

	for _, bar in next, self.Enum.ActionBar do
		local barConfig = self:GetBarConfig(bar)

		for _, button in next, bar.actionButtons or {} do
			self:ConfigureActionButton(button, barConfig)
		end
	end
end

function M:ConfigureActionButton(button, barConfig)
	B:CreateBorder(button, C.BORDER_PADDING)

	button.icon:SetTexCoord(unpack(C.ZOOM_TEX_COORD))
	button.icon:RemoveMaskTexture(button.IconMask)
	button.cooldown:SetAllPoints(button)

	button.Border:SetTexture(nil)
	button.FlyoutBorderShadow:SetTexture(nil)
	button.NormalTexture:SetTexture(nil)
	button.SlotArt:SetTexture(nil)
	button.SlotBackground:SetTexture(nil)

	if type(barConfig) == "table" then
		button.HotKey:SetAlpha((barConfig.showBindingText or (barConfig.showTextWithClutterKey and self.isClutterKeyDown)) and 1 or 0)
		button.Name:SetAlpha((barConfig.showMacroText or (barConfig.showTextWithClutterKey and self.isClutterKeyDown)) and 1 or 0)

		if barConfig.showTextWithClutterKey then
			if not barConfig.showBindingText then
				table.insert(self.text, button.HotKey)
			end

			if not barConfig.showMacroText then
				table.insert(self.text, button.Name)
			end
		end
	end

	if not self:IsHooked(button, "UpdateButtonArt") then
		self:SecureHook(button, "UpdateButtonArt", "ConfigureActionButton")
	end
end

function M:UpdateAllText()
	for _, text in next, self.text do
		self:UpdateText(text)
	end
end

function M:UpdateText(text)
	text:SetAlpha(self.isClutterKeyDown and 1 or 0)
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CLUTTER_STATE_CHANGED(event, state)
	self.isClutterKeyDown = state
	self:UpdateAllText()
end

function M:SIXUI_CONFIG_UPDATE(event, module, config, key, ...)
	if module == self:GetName() then
		self:ConfigureAll()
	end
end
